/* $Id: util_table.h 2392 2007-09-07 22:41:40Z kuhlmann $ */

#ifndef CLIMM_UTIL_TABLE
#define CLIMM_UTIL_TABLE

const char *TableGetMonth (int code);
const char *TableGetLang (UBYTE code);
void        TablePrintLang (void);
const char *TableGetCountry (UWORD code);
const char *TableGetAffiliation (UWORD code);
const char *TableGetPast (UWORD code);
const char *TableGetOccupation (UWORD code);
const char *TableGetInterest (UWORD code);

#endif /* CLIMM_UTIL_TABLE */
